(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(*:Version: Mathematica 2.0 *)
(*:Name: Growth *)
(*:Title: Perturbation Solution Methods for Economic Growth Models *)
(*:Author: Kenneth L. Judd and Sy-Ming Guu, September, 1992 *)
(*:Summary:
  This package provides procedures for computing Taylor and Pade
  approximations to the solutions of optimal growth problems.*)
(*:Keywords: dynamic programming, optimal control, perturbations *)
(*:Requirements: No special system requirements. *)


BeginPackage["Growth`"]

k::usage = "k is the capital stock"

s::usage = "s is the variance of productivity shocks"

Grad::usage =
	"Grad[f[x1,..,xn],{x1,...,xn}] calculates the vector of partial derivatives of f wrt the x's."

cpadefunc::usage =
	"cpadefunc[c[k],k0,n] calculates the (total) degree n Pade approximation of the function c[k] around k0."

ctayfunc::usage =
	"ctayfunc[alpha,gamma,rho,n] calculates the degree n Taylor series expansion of the consumption policy function around the steady state capital stock (where the steady state has been normalized to be 1) when the capital share is alpha, the relative risk aversion is gamma, and the rate of discount is rho."


cstoch::usage =
	"cstoch[alpha,gamma,rho,n] calculates the degree n  two-variable (the variables being deviation from the deterministic steady state and the variance of the productivity shock) Taylor series expansion of the consumption policy function around the zero-variance, steady-state capital stock (which has been normalized to be equal to 1) when the capital share is alpha, the relative risk aversion is gamma, and the rate of discount is rho."


residcontdet::usage =
	"residcontdet[c[k].alpha,gamma,rho] is the Euler equation residual, a function of k, in the deterministic continuous-time growth model if c[k] is the consumption policy function when the capital share is alpha, the relative risk aversion is gamma, and the rate of discount is rho."



residcontstoch::usage =
	"residcontstoch[c[k,s],alpha,gamma,rho] is the Euler equation residual, a function of k and s, in the stochastic continuous-time growth model if c[k,s] is the consumption policy function when the capital share is alpha, the relative risk aversion is gamma, and the rate of discount is rho."


resid::usage =
	"residdiscdet[c[k].alpha,gamma,beta] calculates the Euler equation residual in the deterministic discrete-time growth model if c[k] is the consumption policy function when the capital share is alpha, the relative risk aversion is gamma, and the discount factor is beta."


Begin["`Private`"]


End[]
EndPackage[]



ctayfunc[alpha_,gamma_,rho_,n_] :=
Block[{f,u,kss,cd,c,root,bellman,xbellman},
      f[kk_] = (rho/alpha) kk^alpha;
      u[c_] = c^(1+gamma)/(1+gamma);
      kss = 1;
      c[kk_] := Sum[cd[i] (kk-kss)^i/i!, {i,0,n}];
      bellman[kk_] = - rho*c[kk] - gamma*c[kk]*Derivative[1][c][kk]+
               gamma*f[kk]*Derivative[1][c][kk]+
               c[kk]*Derivative[1][f][kk];
      kss = 1;
      c[kss] = f[kss];
      cd[0] = f[kss];
      xbellman[kk_] = D[bellman[kk],kk];
      root = Solve[xbellman[kss]==0,cd[1]];
      r1=root[[1]][[1]][[2]];
      r2=root[[2]][[1]][[2]];
      If[r1>0,cd[1]=r1,cd[1]=r2];
      For[j=2,j<n+1,j++,
          xbellman[kk_] = D[xbellman[kk],kk];
          root=Solve[xbellman[kss]==0,cd[j]];
          cd[j] = root[[1]][[1]][[2]]
         ];
      Sum[cd[i] (k-kss)^i/i!, {i,0,n}]
     ]


cpadefunc[c_,xpt_,n_] :=
Block[{mpade,npade,numpol,numcoef,denpol,
       dencoef,taylor,pade,c1},
      mpade = Floor[n/2];
      npade = n - mpade;
      c1[x_] = c[x+xpt];
      numpol[x_] = Sum[numcoef[i] x^(i-1),{i,npade+1}];
      denpol[x_] = Sum[dencoef[i] x^(i-1),{i,mpade+1}];
      taylor[x_] = Series[c1[x],{x,0,npade+mpade}];
      diff = numpol[x] - taylor[x] denpol[x];
      dencoef[1] = 1.;
      s1 = Solve[LogicalExpand[diff==0]];
      Do[numcoef[i]=numcoef[i]/.s1[[1]],{i,npade+1}];
      Do[dencoef[i]=dencoef[i]/.s1[[1]],{i,mpade+1}];
      pade[x_]=numpol[x]/denpol[x];
      pade[k-xpt]
      ]

residcontdet[c_,alpha_,gamma_,rho_] := (( rho k^(alpha-1.0)-rho)*c[k]+
          gamma*( (rho/alpha) k^alpha-c[k])*c'[k])/(rho*c[1.0])

cstoch[alpha_, gamma_, rho_,n_] :=
Block[ {kss,f,u,u1,u2,sbell,conss,cons,kk,ss},
   kss=1; 
   f[x_] = rho/alpha x^alpha;
   u[c_] = c^(1+gamma)/(1+gamma);
   u1[c_] = Simplify[u'[c]/u''[c]];
   u2[c_] = Simplify[u'''[c]/u''[c]];

   For[j=0,j<=n,j++,
    For[p=0,p<=n,p++,
     Derivative[j,p][cons][kss,0] = .;
       ];
      ];
   sbell[0,0,kk_,ss_] = (f'[kk] - rho) u1[cons[kk,ss]] + 
     D[cons[kk,ss],kk] (f[kk]-cons[kk,ss]) +
     ss kk kk (u2[cons[kk,ss]] (D[cons[kk,ss],kk])^2 + D[cons[kk,ss],{kk,2}]);
   sbell[1,0,kk_,ss_] = D[sbell[0,0,kk,ss],kk];
   cons[kss,0] = f[kss];
   conss[kk_,ss_] = cons[kss,0];
   root = Solve[sbell[1,0,kss,0]==0,Derivative[1,0][cons][kss,0]];
   r1=root[[1]][[1]][[2]];
   r2=root[[2]][[1]][[2]];
   If[r1>0,rt=r1,rt=r2];
   Derivative[1,0][cons][kss,0] = rt;
   conss[kk_,ss_] = conss[kk,ss] 
                 + Derivative[1,0][cons][kss,0] (kk-kss);
   For[j=2,j<n+1,j++,
     sbell[j,0,kk_,ss_] = D[sbell[j-1,0,kk,ss],kk];
     root = Solve[sbell[j,0,kss,0]==0,Derivative[j,0][cons][kss,0]];
     Derivative[j,0][cons][kss,0] = root[[1]][[1]][[2]];
     conss[kk_,ss_] = conss[kk,ss] + 
        Derivative[j,0][cons][kss,0] (kk-kss)^j / j!
      ];
  For[p=1,p<=n/2,p++,
    sbell[0,p,kk_,ss_] = D[sbell[0,p-1,kk,ss],ss];
    root = Solve[sbell[0,p,kss,0]==0,Derivative[0,p][cons][kss,0]];
    Derivative[0,p][cons][kss,0] = root[[1]][[1]][[2]];
    conss[kk_,ss_] = conss[kk,ss] + 
      Derivative[0,p][cons][kss,0] ss^p / p!;
    For[j=1, j <= n - 2 p,j++,
      sbell[j,p,kk_,ss_] = D[sbell[j-1,p,kk,ss],kk];
      root = Solve[sbell[j,p,kss,0]==0,Derivative[j,p][cons][kss,0]];
      Derivative[j,p][cons][kss,0] = root[[1]][[1]][[2]];
      conss[kk_,ss_] = conss[kk,ss] + 
        Derivative[j,p][cons][kss,0] (kk-kss)^j ss^p/(j! p!)
       ];
     ];
    conss[k,s]
  ]

residcontstoch[c_,alpha_,gamma_,rho_] :=  Block[{u,u1,u2,f,sbells,kss},
   kss=1; 
   f[kk_] = rho/alpha kk^alpha;
   u[x_] = x^(1+gamma)/(1+gamma);
   u1[x_] = Simplify[u'[x]/u''[x]];
   u2[x_] = Simplify[u'''[x]/u''[x]];
   sbells[0,0,kk_,ss_] = (f'[kk] - rho) u1[c[kk,ss]] + 
     D[c[kk,ss],kk] (f[kk]-c[kk,ss]) +
     ss kk kk (u2[c[kk,ss]] (D[c[kk,ss],kk])^2 + D[c[kk,ss],{kk,2}]);
   sbells[0,0,k,s]/(rho u1[c[kss,0]])
]


cdtay[alpha_,gamma_,beta_,n_] :=
Block[{cd,f,a,u,u1,u2,euler1,sol},
 kss=1.; 
 a=kss/(alpha beta kss)^alpha;
 f[kk_]=a kk^alpha;
 u[y_]=y^(1+gamma)/(1+gamma);
 u1[y_]=u'[y];
 u2[kk_]=u'[cd[f[kk-cd[kk]]]];
 cd[kss]=kss*(1-alpha beta);
 euler1[0,kk_]=cd[f[kk-cd[kk]]]*
                (beta f'[kk-cd[kk]])^(1/gamma)-cd[kk];
 Derivative[1][cd][kss] = .;
 euler1[1,kk_]=D[euler1[0,kk],kk];
 euler1[0,kk] = .;
 root=Solve[euler1[1,kss]==0, Derivative[1][cd][kss]];
 r1=root[[1]][[1]][[2]];
 r2=root[[2]][[1]][[2]];
 If[r1>0,rt=r1,rt=r2];
 Derivative[1][cd][kss]=rt;
 For[j=1,j<n,j++,
     Derivative[j][cd][kss] = .;
     euler1[j,kk_]=D[euler1[j-1,kk],kk];
     euler1[j-1,kk] = .;
     sol=Solve[euler1[j,kss]==0,
            Derivative[j][cd][kss]];
     vt=Derivative[j][cd][kss]/.sol[[1]];
     Derivative[j][cd][kss]=vt;
   ];
 Do[cee[ii]=1/(ii!)* 
        Derivative[ii][cd][kss],{ii,1,n-1}];
 Do[cd[kk_]=cd[kss]+
        Sum[cee[iii] (kk-kss)^iii,{iii,1,jj}],
             {jj,1,n-1}];
 cd[k]
]

residdiscdet[c_,alpha_,gamma_,beta_] := Block[{kss,a,f},
 kss=1.; 
 a=kss/(alpha beta kss)^alpha;
 f[kk_]=a kk^alpha;
(c[f[k-c[k]]]*(beta f'[k-c[k]])^(1/gamma)-c[k]) / c[kss]
]

