(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(*
Mathematica code for paper entitled



Gary Anderson
Symbolic Algebra Programming for Analyzing the 
Long Run Dynamics of Economic Models:
An Investigation of Overlapping Generations Models

March 1992

for presentation at
Conference on Computing for the Social Sciences,
May 4-7 1992
University of Michigan
Ann Arbor
Track 2: Research Strategies and Analytic Methods





*)
(*
Analysis of model described in Benhabib and Laroque JET 45
Competitive Cycles in Productive Economies.

Model has capital lagged so as to conform with Anderson/Moore
 conventions on "jump" variables
*)

(*Read in symbolic processing routines for model analysis*)
Print["Load Symbolic Aim Routines"]
<<aim.m
Print["Done"]


(*This is analog of equation 1.1 in 
Benhabib and Laroque
On Competitive Cycles in Productive Economies
Journal of Economic Theory
Volume 45
1988

Capital lagged to conform with Anderson/Moore conventions
on "jump" variables
*)

Print["Defining OLGSystem"]
OLGSystemFunction[ktMinusOne_,ltMinusOne_,qtMinusOne_,
		kt_,lt_,qt_,
		ktPlusOne_,ltPlusOne_,qtPlusOne_]={
(*Consumer maximization*)
D[F[kt,ltPlusOne],kt] D[F[ktMinusOne,lt],lt] Uprime[
			D[F[kt,ltPlusOne],kt] D[F[ktMinusOne,lt],lt] lt]
-Vprime[lt] ,

(*Consumer budget constraint*)
kt + M * qt - D[F[ktMinusOne,lt],lt] lt ,


(*Consumer arbitrage condition*)
qtPlusOne - qt D[F[kt,ltPlusOne],kt] 
}


OLGSystem=OLGSystemFunction[k[t-1],l[t-1],q[t-1],
				k[t],l[t],q[t],
				k[t+1],l[t+1],q[t+1]]
Print["Done"]

Print["Prepare Simplifying Substitutions"]
SimplifyingSubstitutions0={
	D[F[kval_,lval_],lval_]->FL[kval,lval],
	D[F[kval_,lval_],kval_]->FK[kval,lval],
	D[F[kval_,lval_],kval_,kval_]->FKK[kval,lval],
	D[F[kval_,lval_],lval_,lval_]->FLL[kval,lval],
	D[F[kval_,lval_],kval_,lval_]->FKL[kval,lval],
	D[F[kval_,lval_],lval_,kval_]->FKL[kval,lval],
	Uprime'[cval__]->UPP[cval],
	Vprime'[lval__]->VPP[lval],
	Uprime[cval__]->UP[cval],
	Vprime[lval__]->VP[lval]}






(*Substitutions for equations in the fixed point section of the paper*)

SimplifyingSubstitutions1=({
(*wage is marginal product of labor*)
	D[F[kstar,lstar],lstar]->wstar,
(*rate of interest is marginal product of capital*)
	D[F[kstar,lstar],kstar]->rstar,
	D[F[kstar,lstar],kstar,kstar]->FKK,
	D[F[kstar,lstar],lstar,lstar]->FLL,
	D[F[kstar,lstar],kstar,lstar]->FKL,
	D[F[kstar,lstar],lstar,kstar]->FKL,
	Uprime'[cstar_]->UPP,
	Vprime'[lstar_]->VPP,
	Uprime[cstar_]->UP,
	Vprime[lstar_]->VP})/.SimplifyingSubstitutions0
(*These substitutions will facilitate imposing the fixed point constraint*)
fixedPointSubs={
	k[_]->kstar,
	l[_]->lstar,
	q[_]->qstar,
	FK[_,_]->rstar,
	FL[_,_]->wstar,
	asymptotick[_]->kstar,
	asymptoticl[_]->lstar,
	asymptoticq[_]->qstar};
Print["Done"]

(*

CES Production Technology

*)

CES[K_,L_,rho_,delta_,eta_,A_]:=K*(1 - delta) + 
	A/((1 - eta)/K^rho + eta/L^rho)^rho^(-1)


(*
Utility 
*)

Print["Define logarithmic utility functions"]
LogarithmicU[c_]:= Log[c]
LogarithmicV[l_,Lbar_]:= - Log[Lbar -  l]
Print["Done"]



Print["Defining model data structure"]
OLGSystemModel={OLGSystemFunction,{k,l,q},{M},{1,1}}
Print["Done"]

Print["Compute the symbolic H matrix for analyzing fixed points"]
OLGSystemHMatrix=makeSymbolicAsymptoticHMatrix[OLGSystemModel,1]
Print["Done"]


Print["Simplifying HMatrix before applying Aim"];





(*Constant returns to scales assumption provides relationship between
marginal product and elasticity of substitution*)
(*following henderson and quandt page 86 with the observation 
that rstar = 1 we have a great simplification in the relationship 
between output and elasticity of substitutions *)

CRSSubstitutions={FKL->wstar/(sigma (ystar)),
Solve[FKK kstar+ FKL lstar == 0,FKK],
		Solve[FKL kstar+ FLL lstar == 0,FLL]
}//Flatten



	SimplifiedPreAimHMatrix=OLGSystemHMatrix//.
				SimplifyingSubstitutions0//.
				fixedPointSubs//.
				SimplifyingSubstitutions1//.
				CRSSubstitutions/.rstar->1//Simplify


Print["Done"];


Print["Applying Anderson Moore Algorithm"];
{OLGSystemAMatrix,OLGSystemQMatrix,OLGSystemPostAimHMatrix}=
	Apart[stateSpaceTransitionMatrix[SimplifiedPreAimHMatrix]];
Print["Done"];






Print["Determine minimal dimension representation and apply substitutions"]
{MinimalDimensionOLGSystemAMatrix,selector}=
	Factor[Apart[
		eliminateInessentialLags[
			{OLGSystemAMatrix,Range[1,Length[OLGSystemAMatrix]]}]]]


Print["Prepare further simplifying substitutions"];

uPrimeUDoubleFactor=UP kstar wstar + UPP kstar lstar wstar^2;
vDoubleUDoubleFactor=VPP lstar sigma ystar - UPP lstar sigma wstar^2 ystar;
kFactor=Apart[lstar rstar tau/(phi wstar kstar) (-M qstar + wstar lstar)];

moreSubs={
rstar->1,
((uPrimeUDoubleFactor/(kstar  wstar))//Simplify)-> phi,
uPrimeUDoubleFactor->kstar  wstar phi,
vDoubleUDoubleFactor->lstar sigma ystar tau,
kFactor-> lstar  tau/(phi wstar)}

MoreMinimalDimensionOLGSystemAMatrix=MinimalDimensionOLGSystemAMatrix//.
			moreSubs;
MoreMinimalDimensionOLGSystemQMatrix=OLGSystemQMatrix[[
			Range[1,Length[OLGSystemQMatrix]],selector]]//.
								moreSubs;

Print["Done"]


Print["Computing Characteristic Polynomial"]
characteristicPolynomial=Det[
	MoreMinimalDimensionOLGSystemAMatrix - lambda IdentityMatrix[4]]

Print["Done"]


Print["Simplifying Characteristic Polynomial"];
simplifiedCharacteristicPolynomial=
	Collect[
	PolynomialQuotient[
	characteristicPolynomial,-lambda,lambda],lambda]//Simplify;
Print["Done"];

Print["capital output ratio substitutions"]
kySubs={2 kstar sigma-> kstar -kstar ( 1 - 2 sigma),
cstar+kstar->ystar,kstar->kyratio ystar,
lstar wstar -> ystar -kstar,cstar->ystar - kstar,
lstar->cstar/wstar,
tau-> alpha phi wstar/(lstar),
qstar->(cstar - kstar)/M,rstar->1}

Print["simplifying A matrix and characteristic polynomial"]
newMinimalDimensionOLGSystemAMatrix=MoreMinimalDimensionOLGSystemAMatrix//.
						moreSubs//.kySubs//Simplify

newCharacteristicPolynomial=simplifiedCharacteristicPolynomial//.
							kySubs//Simplify
Print["Done"]



(*Compute expressions for bifurcation values*)
Print["Computing Hopf bifurcation values"]
hopfPolynomial = -(lambda - 
	Coefficient[newCharacteristicPolynomial,lambda,0])(
	lambda^2 + 2 aCoeff lambda + 1)

hopfConstraints = Map[# == 0 &, CoefficientList[hopfPolynomial,lambda] -
	 CoefficientList[newCharacteristicPolynomial,lambda]]

allOfHopfACoeff=Reduce[hopfConstraints,aCoeff]//Simplify
Print["Done"];




